/* DVMON.C	Turbo C 1.5 Source for DESQview 2.0 Performance Monitor */
/*
 *    Copyright 1988 Barry A. Burke. All rights, except those
 *    specifically granted herein are reserved by the author. The right
 *    to copy and distribute this material is granted without fee for
 *    any and all non-commercial use. This material specifically may
 *    not be distributed or sold for a fee nor incorporated in whole or
 *    in part into any other product that is distributed or sold for a
 *    fee without specific permission of the author. To obtain special
 *    permission or to report any difficulties with this material
 *    contact:
 *                      Barry A. Burke
 *                      Marketing Insights
 *                      35 Elizabeth Circle
 *                      Framingham, MA  01701
 *
 *    THIS MATERIAL IS DISTRIBUTED "as is" WITHOUT ANY EXPRESSED OR
 *    IMPLIED WARRANTY OR LIABILITY FOR DIRECT, INDIRECT OR
 *    CONSEQUENTIAL DAMAGES.
 *
 *
 * Revision History:
 * 1.0	May 13, 1988	BAB	Initial version sent to Quarterdeck BBoard
 * 1.1	May 16, 1988	BAB	Removed DV Criticial stuff - fixes Bell
 *				problem (better, anyway).
 */

#include <time.h>
#include <bios.h>
#include <stdlib.h> 

/* TurboC System Stuff */
unsigned		_stklen = 128;		/* need smallest stack */
unsigned		_heaplen = 1;		/* smallest heap possbile */

void _setenvp(void){}				/* dummy out _setenvp */
void _setargv(void){}				/* dummy out _setargv */
void setdate(void){}				/* these aren't used, either */
void settime(void){}				/* but TurboC insists on them */

void exit(int c)				/* minimal EXIT() function */
{ _exit(c);}

/* Global Variables */

int	IN_DV = 0;

#define	INCRSECS	1			/* Number of seconds between */
						/* Updates                   */

/* Function Declarations */

static long near LongTicks();
static int near DvGetVersion();
static void near DvPause();
#ifdef NOTDEF
static int near DvGetVideoBuffer();
static void near DvBeginCritical();
static void near DvEndCritical();
#endif

main()
{
	long	goaltime, dummy;
	long	counter, maxcount;
	long	percent;

	DvGetVersion();				/* Find out if we're in DV */
	if ( !IN_DV )
	{
		puts( "This is a DESQview program!" );
		exit(1);
	}

	printf( "DVMON 1.1:\n Calibrating..." );	/* Start calibration loop */

	maxcount = 0;

	goaltime = LongTicks() + 18;
	while ( LongTicks() < goaltime ) {}	/* get to an even-tick boundary */

	goaltime = LongTicks() + 18;
	while ( LongTicks() < goaltime )
	{
		DvPause();			/* Didn't know I could do this */
		++maxcount;
	}

	maxcount = ( maxcount * INCRSECS );
	
	while ( 1 )
	{
		counter = 0;
		goaltime = LongTicks() + (INCRSECS*18L);
		if ( goaltime > 1572000L )	/* Handle midnight wrap */
			goaltime = 1572000L;

		while ( LongTicks() < goaltime )
		{
			DvPause();
			++counter;
		}

		percent = (counter*100) / maxcount;	/* Calc % UNUSED */

		_BH = 0;		/* reposition the cursor to 0,0 */
		_AH = 2;
		_DX = 0;
		__int__(0x10);

		time(&dummy);
		printf( "%s    %3d%% CPU Unused", ctime(&dummy), percent );

/* In order to avoid the appearance we are locked up if somebody tries to
 * type in our window, then switch away, we flush the bios keyboard buffer
 * once a second. This also prevent DESQview's warning bell from running
 * until the next key pressed, tho I don't really know why.
 */

		while ( bioskey( 1 ) )	/* Flush keyboard buffer */
			bioskey( 0 );
	}
}

static long near LongTicks()
{
	return( biostime( 0, 0L ));
}

static int near DvGetVersion()
{
	asm	push	BX
	asm	push	CX
	asm	push	DX

	asm	mov	CX,'DE'			/* Invalid date */
	asm	mov	DX,'SQ'
	asm	mov	AX,2B01H		/* DOS Set Date/DESQ int */
	asm	int	21H

	asm	cmp	AL,0FFH
	asm	je	NO_DV

	asm	mov	BX,AX			/* returned version number */
	asm	mov	IN_DV,1
	asm	jmp	short DVGV_RET

NO_DV:
	asm	sub	AX,AX

DVGV_RET:
	asm	pop	DX
	asm	pop	CX
	asm	pop	BX
}


#ifdef	NOTDEF
static int near DvGetVideoBuffer( VidSeg )
int VidSeg;
{
	asm	push	DI
	asm	push	ES

	asm	mov	ES,VidSeg

	DvGetVersion();

	asm	test	AX,AX
	asm	jz	DVGVB_RET		/* jump if no DESQview */

	asm	mov	AH,0FEH			/* DV get video buffer */
	asm	int	10H

DVGVB_RET:
	asm	mov	AX,ES
	asm	pop	ES
	asm	pop	DI
}
#endif

static void near DvPause()
{
#ifdef NOTDEF				/* Assume in DV for this module */
	asm	cmp	IN_DV,1			/* Are we in DESQview? */
	asm	jne	DVP_RET 
#endif
	asm	mov	AX,101AH
	asm	int	15H
	asm	mov	AX,1000H
	asm	int	15H
	asm	mov	AX,1025H
	asm	int	15H
DVP_RET:
}


#ifdef	NOTDEF
static void near DvBeginCritical()
{
	asm	cmp	IN_DV,1
	asm	jne	DVBC_RET

	asm	mov	AX,101AH
	asm	int	15H
	asm	mov	AX,101BH
	asm	int	15H
	asm	mov	AX,1025H
	asm	int	15H
DVBC_RET:
}

static void near DvEndCritical()
{
	asm	cmp	IN_DV,1
	asm	jne	DVEC_RET

	asm	mov	AX,101AH
	asm	int	15H
	asm	mov	AX,101CH
	asm	int	15H
	asm	mov	AX,1025H
	asm	int	15H

DVEC_RET:
}
#endif	/* NOTDEF */
